package gs.log.v0

/**
  * Interface for emitting logs. This and [[Log]] are the primary types that
  * users of this library will interact with.
  */
trait Logger[F[_]]:
  def trace(log: => Log): F[Unit]
  def debug(log: => Log): F[Unit]
  def info(log: => Log): F[Unit]
  def warn(log: => Log): F[Unit]
  def error(log: => Log): F[Unit]
  def fatal(log: => Log): F[Unit]
  def name(): Logger.Name
  def isLevelEnabled(level: LogLevel): Boolean

object Logger:

  /** Opaque type (String) used to represent the descriptive name of a logger. 
   *  If a logger has a name, logs _may_ include that name in their output (if 
   *  configured to do so).
    */
  opaque type Name = String

  object Name:

    def apply(loggerName: String): Name = loggerName

    given CanEqual[Name, Name] = CanEqual.derived

    extension (name: Name) def str(): String = name

  end Name

end Logger
